@@ -176,7 +176,8 @@ LPTSTR Help_FindCommand_Search (UNIXUTIL *puu, LPTSTR pszKeyword)
          ++pszKeyword;
 
       // find the end of this word
-      for (LPTSTR pszNext = pszKeyword; *pszNext && !iswhite(*pszNext); )
+      LPTSTR pszNext;
+      for (pszNext = pszKeyword; *pszNext && !iswhite(*pszNext); )
          ++pszNext;
       if (!*pszNext)  // last word?  Gotta use it.
          break;
@@ -215,7 +216,7 @@ DWORD NextSearch (int &ii)
 BOOL Help_FindCommand_OnOK (HWND hDlg)
 {
    HWND hCombo = GetDlgItem (hDlg, IDC_FIND_COMMAND);
-   int iiDisplay = -1;
+   size_t iiDisplay = -1;
 
    UNIXUTIL uu = uuUNSPECIFIED;
    TCHAR szText[ cchRESOURCE ];
@@ -409,7 +410,7 @@ void Help_FindError_Shrink (HWND hDlg, BOOL fShrink)
 BOOL CALLBACK Help_About_DlgProc (HWND hDlg, UINT msg, WPARAM wp, LPARAM lp);
 void Help_About_OnInitDialog (HWND hDlg);
 void Help_About_OnSysCommand (HWND hDlg, int &cmd);
-LONG procAbout;
+UINT_PTR procAbout;
 
 void Help_About (void)
 {
@@ -464,15 +465,15 @@ BOOL CALLBACK Help_About_Proc (HWND hDlg, UINT msg, WPARAM wp, LPARAM lp)
          SetTimer (GetParent(hDlg), 1000, 1000/8, NULL);
          }
       }
-   return CallWindowProc ((WNDPROC)procAbout, hDlg, msg, wp, lp);
+   return CallWindowProc ((WNDPROC)procAbout, hDlg, msg, wp, lp)?TRUE:FALSE;
 }
 
 
 void Help_About_OnInitDialog (HWND hDlg)
 {
    HWND hAbout = GetDlgItem (hDlg, IDOK);
-   procAbout = (LONG)GetWindowLong (hAbout, GWL_WNDPROC);
-   SetWindowLong (hAbout, GWL_WNDPROC, (LONG)Help_About_Proc);
+   procAbout = (UINT_PTR)GetWindowLongPtr (hAbout, GWLP_WNDPROC);
+   SetWindowLongPtr (hAbout, GWLP_WNDPROC, (LONG_PTR)Help_About_Proc);
 
    LPTSTR pszText = FormatString (IDS_HELPABOUT_DESC1);
    SetDlgItemText (hDlg, IDC_HELPABOUT_DESC, pszText);
@@ -485,17 +486,18 @@ void Help_About_OnSysCommand (HWND hDlg, int &cmd)
    DWORD dw;
    TCHAR szSys[cchRESOURCE];
    TCHAR szSys2[cchRESOURCE];
+   LPTSTR psz;
    GetDlgItemText (hDlg, 0x07E6, szSys, cchRESOURCE);
    if ((dw = NextSearch (cmd)) != 0)
       {
-      for (LPTSTR psz = &szSys[ lstrlen(szSys)-1 ]; *(psz-1) != TEXT('\n'); --psz);
+      for (psz = &szSys[ lstrlen(szSys)-1 ]; *(psz-1) != TEXT('\n'); --psz);
       lstrcpy (szSys2, psz);
       wsprintf (psz, TEXT("%c%s"), (TCHAR)dw, szSys2);
       SetDlgItemText (hDlg, 0x07E6, szSys);
       }
    else // (dw == 0)
       {
-      for (LPTSTR psz = szSys; *psz && (*psz != TEXT('\n')); ++psz);
+      for (psz = szSys; *psz && (*psz != TEXT('\n')); ++psz);
       wsprintf (szSys2, TEXT("%s\n"), 1+psz);
       SetDlgItemText (hDlg, 0x07E6, szSys2);
       }
