@@ -7,6 +7,9 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#include <winsock2.h>
+#include <ws2tcpip.h>
+
 extern "C" {
 #include <afs/param.h>
 #include <afs/stds.h>
@@ -14,6 +17,7 @@ extern "C" {
 
 #include <windows.h>
 #include <stdlib.h>
+#include <WINNT\afsreg.h>
 #include "pagesize.h"
 
 
