@@ -7,6 +7,9 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#include <winsock2.h>
+#include <ws2tcpip.h>
+
 extern "C" {
 #include <afs\stds.h>
 #include <afs\param.h>
@@ -19,7 +22,7 @@ extern "C" {
 }
 
 #include "afscreds.h"
-
+#include <WINNT\afsreg.h>
 
 /*
  * DEFINITIONS ________________________________________________________________
@@ -155,7 +158,8 @@ void GetGatewayName (LPTSTR pszGateway)
 {
    *pszGateway = TEXT('\0');
    HKEY hk;
-   if (RegOpenKey (HKEY_LOCAL_MACHINE, TEXT("System\\CurrentControlSet\\Services\\TransarcAFSDaemon\\Parameters"), &hk) == 0)
+   if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, TEXT(AFSREG_CLT_SVC_PARAM_SUBKEY), 0,
+                      (IsWow64()?KEY_WOW64_64KEY:0)|KEY_QUERY_VALUE, &hk) == 0)
       {
       DWORD dwSize = MAX_PATH;
       DWORD dwType = REG_SZ;
@@ -213,7 +217,7 @@ BOOL IsServicePersistent (void)
    if ((hManager = OpenSCManager (NULL, NULL, GENERIC_READ)) != NULL)
       {
       SC_HANDLE hService;
-      if ((hService = OpenService (hManager, TEXT("TransarcAFSDaemon"), GENERIC_READ)) != NULL)
+      if ((hService = OpenService (hManager, TEXT(AFSREG_CLT_SVC_NAME), GENERIC_READ)) != NULL)
          {
          DWORD dwSize = sizeof(Config);
          QueryServiceConfig (hService, (QUERY_SERVICE_CONFIG*)&Config, sizeof(Config), &dwSize);
@@ -237,7 +241,8 @@ BOOL IsServiceConfigured (void)
       {
       rc = TRUE;
       }
-   else if (RegOpenKey (HKEY_LOCAL_MACHINE, TEXT("System\\CurrentControlSet\\Services\\TransarcAFSDaemon\\Parameters"), &hk) == 0)
+   else if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, TEXT(AFSREG_CLT_SVC_PARAM_SUBKEY), 0,
+                           (IsWow64()?KEY_WOW64_64KEY:0)|KEY_QUERY_VALUE, &hk) == 0)
       {
       TCHAR szCell[ MAX_PATH ];
       DWORD dwSize = sizeof(szCell);
@@ -312,7 +317,8 @@ int GetCurrentCredentials (void)
          // credentials within a certain cell under the certain name.
          // Stick that knowledge in our g.aCreds array.
          //
-         for (size_t iCreds = 0; iCreds < g.cCreds; ++iCreds)
+         size_t iCreds;
+         for (iCreds = 0; iCreds < g.cCreds; ++iCreds)
             {
             if (!lstrcmpi (g.aCreds[ iCreds ].szCell, szCell))
                break;
@@ -450,7 +456,8 @@ int GetDefaultCell (LPTSTR pszCell)
         int rc;
         HKEY hk;
 
-        if (RegOpenKey (HKEY_CURRENT_USER, REGSTR_PATH_OPENAFS_CLIENT, &hk) == 0)
+        if (RegOpenKeyEx (HKEY_CURRENT_USER, TEXT(AFSREG_USER_OPENAFS_SUBKEY), 0,
+                        (IsWow64()?KEY_WOW64_64KEY:0)|KEY_QUERY_VALUE, &hk) == 0)
         {
             DWORD dwSize = sizeof(szCellA);
             DWORD dwType = REG_SZ;
