@@ -1,8 +1,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/afs/FBSD/osi_vfsops.c,v 1.21.4.1 2008/08/26 14:01:33 shadow Exp $");
 
 #include <afs/sysincludes.h>	/* Standard vendor system headers */
 #include <afsincludes.h>	/* Afs-based standard headers */
@@ -25,7 +23,6 @@ int afs_pbuf_freecnt = -1;
 #define	THREAD_OR_PROC struct proc *p
 #endif
 
-extern int afs3_syscall();
 extern int Afs_xsetgroups();
 extern int afs_xioctl();
 
@@ -166,7 +163,9 @@ afs_unmount(struct mount *mp, int flags, THREAD_OR_PROC)
 }
 
 int
-#if defined(AFS_FBSD60_ENV)
+#if defined(AFS_FBSD80_ENV)
+afs_root(struct mount *mp, int flags, struct vnode **vpp)
+#elif defined(AFS_FBSD60_ENV)
 afs_root(struct mount *mp, int flags, struct vnode **vpp, struct thread *td)
 #elif defined(AFS_FBSD53_ENV)
 afs_root(struct mount *mp, struct vnode **vpp, struct thread *td)
@@ -178,10 +177,10 @@ afs_root(struct mount *mp, struct vnode **vpp)
     struct vrequest treq;
     register struct vcache *tvp = 0;
 #ifdef AFS_FBSD50_ENV
-#ifndef AFS_FBSD53_ENV
+#if !defined(AFS_FBSD53_ENV) || defined(AFS_FBSD80_ENV)
     struct thread *td = curthread;
 #endif
-    struct ucred *cr = td->td_ucred;
+    struct ucred *cr = osi_curcred();
 #else
     struct proc *p = curproc;
     struct ucred *cr = p->p_cred->pc_ucred;
@@ -190,7 +189,7 @@ afs_root(struct mount *mp, struct vnode **vpp)
     AFS_GLOCK();
     AFS_STATCNT(afs_root);
     crhold(cr);
-    if (afs_globalVp && (afs_globalVp->states & CStatd)) {
+    if (afs_globalVp && (afs_globalVp->f.states & CStatd)) {
 	tvp = afs_globalVp;
 	error = 0;
     } else {
