@@ -7,18 +7,15 @@
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-#define	IGNORE_STDS_H
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/gtx/input.c,v 1.7.14.1 2007/11/26 21:08:42 shadow Exp $");
+#include <stdlib.h>
 
 #ifdef AFS_HPUX_ENV
 #include <sys/types.h>
 #endif
 #include <lwp.h>
-
 #include "gtxobjects.h"
 #include "gtxwindows.h"
 #include "gtxcurseswin.h"
@@ -29,9 +26,11 @@ RCSID
 
 
 /* process input */
-gtx_InputServer(awin)
-     register struct gwin *awin;
+void *
+gtx_InputServer(void *param)
 {
+    struct gwin *awin = (struct gwin *) param;
+    
     register int tc;
     register int code;
     register struct gtx_frame *tframe;
@@ -65,9 +64,8 @@ gtx_InputServer(awin)
 }
 
 struct gwin *
-gtx_Init(astartInput, atype)
-     int atype;			/* type of window to create */
-     int astartInput;
+gtx_Init(int astartInput, 
+	 int atype)			/* type of window to create */
 {
     PROCESS junk;
     struct onode_initparams oi_params;	/* object init params */
