@@ -109,13 +109,10 @@ ka_UserAuthenticateGeneral2(afs_int32 flags, char *name, char *instance,
     code = cm_SearchCellRegistry(1, realm, fullRealm, NULL, ka_AddHostProc, NULL);
     if (code && code != CM_ERROR_FORCE_DNS_LOOKUP)
         code = cm_SearchCellFile(realm, fullRealm, ka_AddHostProc, NULL);
-
-#ifdef AFS_AFSDB_ENV
     if (code) {
 	code =
 	    cm_SearchCellByDNS(realm, fullRealm, &ttl, ka_AddHostProc, NULL);
     }
-#endif
     if (code) {
 	*reasonP = "specified realm is unknown";
 	return (code);
@@ -716,7 +713,7 @@ send_to_kdc(pkt, rpkt)
     struct sockaddr_in to;
     int timeAvail, timePerIter, numIters;
 
-    memset((char *)&to, 0, sizeof(to));
+    memset(&to, 0, sizeof(to));
     if ((f = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
 	if (krb_debug)
 	    fprintf(stderr, "%s: Can't open socket\n", prog);
