@@ -14,8 +14,6 @@
 #include <afs/param.h>
 #endif
 
-RCSID
-    ("$Header: /cvs/openafs/src/ptserver/ptuser.c,v 1.25.2.11 2007/10/30 15:16:43 shadow Exp $");
 
 #if defined(UKERNEL)
 #include "afs/sysincludes.h"
@@ -30,6 +28,7 @@ RCSID
 #include "afs/ptclient.h"
 #include "afs/ptuser.h"
 #include "afs/pterror.h"
+#include "afs/com_err.h"
 #else /* defined(UKERNEL) */
 #include <afs/stds.h>
 #include <ctype.h>
@@ -63,15 +62,17 @@ pr_Initialize(IN afs_int32 secLevel, IN const char *confDir, IN char *cell)
 {
     afs_int32 code;
     struct rx_connection *serverconns[MAXSERVERS];
-    struct rx_securityClass *sc[3];
+    struct rx_securityClass *sc = NULL;
     static struct afsconf_dir *tdir = (struct afsconf_dir *)NULL;	/* only do this once */
     static char tconfDir[100] = "";
     static char tcell[64] = "";
-    struct ktc_token ttoken;
     afs_int32 scIndex;
+    afs_int32 secFlags;
     static struct afsconf_cell info;
     afs_int32 i;
+#if !defined(UKERNEL)
     char cellstr[64];
+#endif
     afs_int32 gottdir = 0;
     afs_int32 refresh = 0;
 
@@ -91,11 +92,12 @@ pr_Initialize(IN afs_int32 secLevel, IN const char *confDir, IN char *cell)
 	if (!tdir) {
 	    if (confDir && strcmp(confDir, ""))
 		fprintf(stderr,
-			"libprot: Could not open configuration directory: %s.\n",
-			confDir);
+			"%s: Could not open configuration directory: %s.\n",
+			whoami, confDir);
             else
 		fprintf(stderr,
-			"libprot: No configuration directory specified.\n");
+			"%s: No configuration directory specified.\n",
+			whoami);
 	    return -1;
 	}
         gottdir = 1;
@@ -166,64 +168,48 @@ pr_Initialize(IN afs_int32 secLevel, IN const char *confDir, IN char *cell)
 	return code;
     }
 
-    scIndex = secLevel;
-    sc[0] = 0;
-    sc[1] = 0;
-    sc[2] = 0;
     /* Most callers use secLevel==1, however, the fileserver uses secLevel==2
      * to force use of the KeyFile.  secLevel == 0 implies -noauth was
      * specified. */
-    if ((secLevel == 2) && (afsconf_GetLatestKey(tdir, 0, 0) == 0)) {
-	/* If secLevel is two assume we're on a file server and use
-	 * ClientAuthSecure if possible. */
-	code = afsconf_ClientAuthSecure(tdir, &sc[2], &scIndex);
-	if (code)
-	    fprintf(stderr,
-		    "libprot: clientauthsecure returns %d %s"
-		    " (so trying noauth)\n", code, afs_error_message(code));
-	if (code)
-	    scIndex = 0;	/* use noauth */
-	if (scIndex != 2)
-	    /* if there was a problem, an unauthenticated conn is returned */
-	    sc[scIndex] = sc[2];
+    if (secLevel == 2) {
+	code = afsconf_GetLatestKey(tdir, 0, 0);
+	if (code) {
+	    afs_com_err(whoami, code, "(getting key from local KeyFile)\n");
+	} else {
+	    /* If secLevel is two assume we're on a file server and use
+	     * ClientAuthSecure if possible. */
+	    code = afsconf_ClientAuthSecure(tdir, &sc, &scIndex);
+	    if (code)
+		afs_com_err(whoami, code, "(calling client secure)\n");
+        }
     } else if (secLevel > 0) {
-	struct ktc_principal sname;
-	strcpy(sname.cell, info.name);
-	sname.instance[0] = 0;
-	strcpy(sname.name, "afs");
-	code = ktc_GetToken(&sname, &ttoken, sizeof(ttoken), NULL);
-	if (code)
-	    scIndex = 0;
-	else {
-	    if (ttoken.kvno >= 0 && ttoken.kvno <= 256)
-		/* this is a kerberos ticket, set scIndex accordingly */
-		scIndex = 2;
-	    else {
-		fprintf(stderr,
-			"libprot: funny kvno (%d) in ticket, proceeding\n",
-			ttoken.kvno);
-		scIndex = 2;
-	    }
-	    sc[2] =
-		rxkad_NewClientSecurityObject(rxkad_clear, &ttoken.sessionKey,
-					      ttoken.kvno, ttoken.ticketLen,
-					      ttoken.ticket);
+	secFlags = 0;
+	if (secLevel > 1)
+	    secFlags |= AFSCONF_SECOPTS_ALWAYSENCRYPT;
+
+	code = afsconf_ClientAuthToken(&info, secFlags, &sc, &scIndex, NULL);
+	if (code) {
+	    afs_com_err(whoami, code, "(getting token)");
+	    if (secLevel > 1)
+		return code;
 	}
     }
 
-    if (scIndex == 1)
-	return PRBADARG;
-    if ((scIndex == 0) && (sc[0] == 0))
-	sc[0] = rxnull_NewClientSecurityObject();
-    if ((scIndex == 0) && (secLevel != 0))
-	afs_com_err(whoami, code,
-		"Could not get afs tokens, running unauthenticated.");
+    if (sc == NULL) {
+	sc = rxnull_NewClientSecurityObject();
+        scIndex = RX_SECIDX_NULL;
+    }
+
+    if ((scIndex == RX_SECIDX_NULL) && (secLevel != 0))
+	fprintf(stderr,
+		"%s: Could not get afs tokens, running unauthenticated\n",
+		whoami);
 
     memset(serverconns, 0, sizeof(serverconns));	/* terminate list!!! */
     for (i = 0; i < info.numServers; i++)
 	serverconns[i] =
 	    rx_NewConnection(info.hostAddr[i].sin_addr.s_addr,
-			     info.hostAddr[i].sin_port, PRSRV, sc[scIndex],
+			     info.hostAddr[i].sin_port, PRSRV, sc,
 			     scIndex);
 
     code = ubik_ClientInit(serverconns, &pruclient);
@@ -233,12 +219,12 @@ pr_Initialize(IN afs_int32 secLevel, IN const char *confDir, IN char *cell)
     }
     lastLevel = scIndex;
 
-    code = rxs_Release(sc[scIndex]);
+    code = rxs_Release(sc);
     return code;
 }
 
 int
-pr_End()
+pr_End(void)
 {
     int code = 0;
 
@@ -325,7 +311,7 @@ pr_AddToGroup(char *user, char *group)
     idlist lids;
 
     lnames.namelist_len = 2;
-    lnames.namelist_val = (prname *) malloc(2 * PR_MAXNAMELEN);
+    lnames.namelist_val = malloc(2 * PR_MAXNAMELEN);
     strncpy(lnames.namelist_val[0], user, PR_MAXNAMELEN);
     strncpy(lnames.namelist_val[1], group, PR_MAXNAMELEN);
     lids.idlist_val = 0;
@@ -345,8 +331,8 @@ pr_AddToGroup(char *user, char *group)
   done:
     if (lnames.namelist_val)
 	free(lnames.namelist_val);
-    if (lids.idlist_val)
-	free(lids.idlist_val);
+
+    xdr_free((xdrproc_t) xdr_idlist, &lids);
     return code;
 }
 
@@ -358,7 +344,7 @@ pr_RemoveUserFromGroup(char *user, char *group)
     idlist lids;
 
     lnames.namelist_len = 2;
-    lnames.namelist_val = (prname *) malloc(2 * PR_MAXNAMELEN);
+    lnames.namelist_val = malloc(2 * PR_MAXNAMELEN);
     strncpy(lnames.namelist_val[0], user, PR_MAXNAMELEN);
     strncpy(lnames.namelist_val[1], group, PR_MAXNAMELEN);
     lids.idlist_val = 0;
@@ -378,10 +364,10 @@ pr_RemoveUserFromGroup(char *user, char *group)
   done:
     if (lnames.namelist_val)
 	free(lnames.namelist_val);
-    if (lids.idlist_val)
-	free(lids.idlist_val);
-    return code;
 
+    xdr_free((xdrproc_t) xdr_idlist, &lids);
+
+    return code;
 }
 
 int
@@ -406,13 +392,13 @@ pr_SNameToId(char name[PR_MAXNAMELEN], afs_int32 *id)
     lids.idlist_len = 0;
     lids.idlist_val = 0;
     lnames.namelist_len = 1;
-    lnames.namelist_val = (prname *) malloc(PR_MAXNAMELEN);
+    lnames.namelist_val = malloc(PR_MAXNAMELEN);
     stolower(name);
     strncpy(lnames.namelist_val[0], name, PR_MAXNAMELEN);
     code = ubik_PR_NameToID(pruclient, 0, &lnames, &lids);
     if (lids.idlist_val) {
 	*id = *lids.idlist_val;
-	free(lids.idlist_val);
+	xdr_free((xdrproc_t) xdr_idlist, &lids);
     }
     if (lnames.namelist_val)
 	free(lnames.namelist_val);
@@ -436,17 +422,20 @@ pr_SIdToName(afs_int32 id, char name[PR_MAXNAMELEN])
     register afs_int32 code;
 
     lids.idlist_len = 1;
-    lids.idlist_val = (afs_int32 *) malloc(sizeof(afs_int32));
+    lids.idlist_val = malloc(sizeof(afs_int32));
     *lids.idlist_val = id;
     lnames.namelist_len = 0;
     lnames.namelist_val = 0;
     code = ubik_PR_IDToName(pruclient, 0, &lids, &lnames);
-    if (lnames.namelist_val) {
+
+    if (lnames.namelist_val)
 	strncpy(name, lnames.namelist_val[0], PR_MAXNAMELEN);
-	free(lnames.namelist_val);
-    }
+
     if (lids.idlist_val)
 	free(lids.idlist_val);
+
+    xdr_free((xdrproc_t) xdr_namelist, &lnames);
+
     return code;
 }
 
@@ -541,12 +530,14 @@ pr_ListOwned(afs_int32 oid, namelist *lnames, afs_int32 *moreP)
 		oid);
 	*moreP = 0;
     }
-    lids = (idlist *) & alist;
+    lids = (idlist *) &alist;
     code = pr_IdToName(lids, lnames);
+
+    xdr_free((xdrproc_t) xdr_prlist, &alist);
+
     if (code)
 	return code;
-    if (alist.prlist_val)
-	free(alist.prlist_val);
+
     return PRSUCCESS;
 }
 
@@ -567,12 +558,13 @@ pr_IDListMembers(afs_int32 gid, namelist *lnames)
 	fprintf(stderr, "membership list for id %d exceeds display limit\n",
 		gid);
     }
-    lids = (idlist *) & alist;
+    lids = (idlist *) &alist;
     code = pr_IdToName(lids, lnames);
+
+    xdr_free((xdrproc_t) xdr_prlist, &alist);
+
     if (code)
 	return code;
-    if (alist.prlist_val)
-	free(alist.prlist_val);
     return PRSUCCESS;
 }
 
@@ -691,7 +683,7 @@ pr_IsAMemberOf(char *uname, char *gname, afs_int32 *flag)
     stolower(uname);
     stolower(gname);
     lnames.namelist_len = 2;
-    lnames.namelist_val = (prname *) malloc(2 * PR_MAXNAMELEN);
+    lnames.namelist_val = malloc(2 * PR_MAXNAMELEN);
     strncpy(lnames.namelist_val[0], uname, PR_MAXNAMELEN);
     strncpy(lnames.namelist_val[1], gname, PR_MAXNAMELEN);
     lids.idlist_val = 0;
@@ -700,8 +692,7 @@ pr_IsAMemberOf(char *uname, char *gname, afs_int32 *flag)
     if (code) {
 	if (lnames.namelist_val)
 	    free(lnames.namelist_val);
-	if (lids.idlist_val)
-	    free(lids.idlist_val);
+	xdr_free((xdrproc_t) xdr_idlist, &lids);
 	return code;
     }
     code =
@@ -709,8 +700,7 @@ pr_IsAMemberOf(char *uname, char *gname, afs_int32 *flag)
 		  lids.idlist_val[1], flag);
     if (lnames.namelist_val)
 	free(lnames.namelist_val);
-    if (lids.idlist_val)
-	free(lids.idlist_val);
+    xdr_free((xdrproc_t) xdr_idlist, &lids);
     return code;
 }
 
