@@ -18,8 +18,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/vfsck/utilities.c,v 1.5.2.1 2004/11/09 17:18:48 shadow Exp $");
 
 #include <sys/param.h>
 #define VICE			/* allow us to put our changes in at will */
@@ -62,7 +60,11 @@ RCSID
 #include <sys/ino.h>
 #include <sys/signal.h>
 #define	DIRSIZ_MACRO
+#ifdef HAVE_USR_OLD_USR_INCLUDE_NDIR_H
+#include </usr/old/usr/include/ndir.h>
+#else
 #include <ndir.h>
+#endif
 #else
 #include <sys/dir.h>
 #endif
@@ -71,6 +73,7 @@ RCSID
 #endif /* AFS_OSF_ENV */
 
 #include <stdio.h>
+#include <string.h>
 #include <ctype.h>
 #include "fsck.h"
 
@@ -89,7 +92,7 @@ long diskreads, totalreads;	/* Disk cache statistics */
 long lseek();
 #endif
 char *malloc();
-#if	defined(AFS_SUN_ENV) || defined(AFS_DEC_ENV)
+#if	defined(AFS_SUN_ENV)
 extern int iscorrupt;
 #endif
 #ifdef	AFS_SUN_ENV
@@ -149,7 +152,7 @@ reply(question)
     printf("\n");
     if (!persevere && (nflag || fswritefd < 0)) {
 	printf("%s? no\n\n", question);
-#if	defined(AFS_SUN_ENV) || defined(AFS_DEC_ENV)
+#if	defined(AFS_SUN_ENV) 
 	iscorrupt = 1;		/* known to be corrupt */
 #endif
 	return (0);
@@ -172,7 +175,7 @@ reply(question)
 #ifdef	AFS_HPUX_ENV
     fixed = 0;
 #endif
-#if	defined(AFS_SUN_ENV) || defined(AFS_DEC_ENV)
+#if	defined(AFS_SUN_ENV)
     iscorrupt = 1;		/* known to be corrupt */
 #endif
     return (0);
@@ -282,9 +285,6 @@ flush(fd, bp)
 	pfatal("WRITING %sZERO'ED BLOCK %d TO DISK\n",
 	       (bp->b_errs == bp->b_size / dev_bsize) ? "" : "PARTIALLY ",
 	       bp->b_bno);
-#ifdef	AFS_DEC_ENV
-	iscorrupt = 1;
-#endif
     }
     bp->b_dirty = 0;
     bp->b_errs = 0;
@@ -591,7 +591,7 @@ getpathname(namebuf, curdir, ino)
 	strcpy(namebuf, "?");
 	return;
     }
-    memset((char *)&idesc, 0, sizeof(struct inodesc));
+    memset(&idesc, 0, sizeof(struct inodesc));
     idesc.id_type = DATA;
     cp = &namebuf[BUFSIZ - 1];
     *cp = '\0';
