@@ -15,14 +15,6 @@
 	Institution:	The Information Technology Center, Carnegie-Mellon University
 
  */
-#ifdef notdef
-
-/* All this is going away in early 1989 */
-int newVLDB;			/* Compatibility flag */
-
-#endif
-static int newVLDB = 1;
-
 
 #ifndef AFS_PTHREAD_ENV
 #define USUAL_PRIORITY (LWP_MAX_PRIORITY - 2)
@@ -43,8 +35,6 @@ static int newVLDB = 1;
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/vol/fssync-client.c,v 1.1.4.3 2008/02/04 18:51:39 shadow Exp $");
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -85,11 +75,6 @@ RCSID
 
 /*@printflike@*/ extern void Log(const char *format, ...);
 
-#ifdef osi_Assert
-#undef osi_Assert
-#endif
-#define osi_Assert(e) (void)(e)
-
 extern int LogLevel;
 
 static SYNC_client_state fssync_state = 
@@ -103,7 +88,7 @@ static SYNC_client_state fssync_state =
 
 #ifdef AFS_PTHREAD_ENV
 static pthread_mutex_t vol_fsync_mutex;
-static volatile vol_fsync_mutex_init = 0;
+static volatile int vol_fsync_mutex_init = 0;
 #define VFSYNC_LOCK \
     assert(pthread_mutex_lock(&vol_fsync_mutex) == 0)
 #define VFSYNC_UNLOCK \
