@@ -16,8 +16,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/vol/purge.c,v 1.12.2.5 2008/06/12 19:18:49 shadow Exp $");
 
 #include <stdio.h>
 #ifdef AFS_NT40_ENV
@@ -27,10 +25,12 @@ RCSID
 #include <sys/param.h>
 #include <sys/file.h>
 #include <sys/time.h>
+#include <unistd.h>
 #endif
 #include <string.h>
 #include <sys/stat.h>
 #include <afs/assert.h>
+#include <afs/afsutil.h>
 
 #include <rx/xdr.h>
 #include "afs/afsint.h"
@@ -52,10 +52,15 @@ RCSID
 /* forward declarations */
 static int ObliterateRegion(Volume * avp, VnodeClass aclass, StreamHandle_t * afile,
 			    afs_int32 * aoffset);
+#if 0
 static void PurgeIndex(Volume * vp, VnodeClass class);
+static void PurgeHeader(Volume * vp);
+#endif 
+
 static void PurgeIndex_r(Volume * vp, VnodeClass class);
 static void PurgeHeader_r(Volume * vp);
-static void PurgeHeader(Volume * vp);
+
+/*@printflike@*/ extern void Log(const char *format, ...);
 
 /* No lock needed. Only the volserver will call this, and only one transaction
  * can have a given volume (volid/partition pair) in use at a time 
@@ -64,7 +69,11 @@ void
 VPurgeVolume(Error * ec, Volume * vp)
 {
     struct DiskPartition64 *tpartp = vp->partition;
-    char purgePath[MAXPATHLEN];
+    VolumeId volid, parent;
+    afs_int32 code;
+
+    volid = V_id(vp);
+    parent = V_parentId(vp);
 
     /* so VCheckDetach doesn't try to update the volume header and
      * dump spurious errors into the logs */
@@ -74,13 +83,17 @@ VPurgeVolume(Error * ec, Volume * vp)
      * volume header. This routine can, under some circumstances, be called
      * when two volumes with the same id exist on different partitions.
      */
-    (void)afs_snprintf(purgePath, sizeof purgePath, "%s/%s",
-		       VPartitionPath(vp->partition),
-		       VolumeExternalName(V_id(vp)));
     PurgeIndex_r(vp, vLarge);
     PurgeIndex_r(vp, vSmall);
     PurgeHeader_r(vp);
-    unlink(purgePath);
+
+    code = VDestroyVolumeDiskHeader(tpartp, volid, parent);
+    if (code) {
+	Log("VPurgeVolume: Error %ld when destroying volume %lu header\n",
+	    afs_printable_int32_ld(code),
+	    afs_printable_uint32_lu(volid));
+    }
+
     /*
      * Call the fileserver to break all call backs for that volume
      */
@@ -159,6 +172,7 @@ ObliterateRegion(Volume * avp, VnodeClass aclass, StreamHandle_t * afile,
     return -1;
 }
 
+#if 0
 static void
 PurgeIndex(Volume * vp, VnodeClass class)
 {
@@ -166,6 +180,7 @@ PurgeIndex(Volume * vp, VnodeClass class)
     PurgeIndex_r(vp, class);
     VOL_UNLOCK;
 }
+#endif
 
 static void
 PurgeIndex_r(Volume * vp, VnodeClass class)
@@ -197,6 +212,7 @@ PurgeIndex_r(Volume * vp, VnodeClass class)
     FDH_CLOSE(fdP);
 }
 
+#if 0
 static void
 PurgeHeader(Volume * vp)
 {
@@ -204,6 +220,7 @@ PurgeHeader(Volume * vp)
     PurgeHeader_r(vp);
     VOL_UNLOCK;
 }
+#endif
 
 static void
 PurgeHeader_r(Volume * vp)
